package sun.management;

import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.io.FileNotFoundException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.io.IOException;
import sun.management.jdp.JdpException;
import java.net.UnknownHostException;
import java.net.InetAddress;
import sun.management.jdp.JdpController;
import java.lang.management.ManagementFactory;
import sun.management.jmxremote.ConnectorBootstrap;
import sun.misc.VMSupport;
import javax.management.remote.JMXConnectorServer;
import java.util.ResourceBundle;
import java.util.Properties;

public class Agent
{
    private static Properties mgmtProps;
    private static ResourceBundle messageRB;
    private static final String CONFIG_FILE = "com.sun.management.config.file";
    private static final String SNMP_PORT = "com.sun.management.snmp.port";
    private static final String JMXREMOTE = "com.sun.management.jmxremote";
    private static final String JMXREMOTE_PORT = "com.sun.management.jmxremote.port";
    private static final String RMI_PORT = "com.sun.management.jmxremote.rmi.port";
    private static final String ENABLE_THREAD_CONTENTION_MONITORING = "com.sun.management.enableThreadContentionMonitoring";
    private static final String LOCAL_CONNECTOR_ADDRESS_PROP = "com.sun.management.jmxremote.localConnectorAddress";
    private static final String SNMP_ADAPTOR_BOOTSTRAP_CLASS_NAME = "sun.management.snmp.AdaptorBootstrap";
    private static final String JDP_DEFAULT_ADDRESS = "224.0.23.178";
    private static final int JDP_DEFAULT_PORT = 7095;
    private static JMXConnectorServer jmxServer;
    
    public Agent() {
    }
    
    private static Properties parseString(String s) {
        Properties properties = new Properties();
        if (s != null && !s.trim().equals("")) {
            String[] split = s.split(",");
            for (int length = split.length, i = 0; i < length; ++i) {
                String[] split2 = split[i].split("=", 2);
                String trim = split2[0].trim();
                String value = (split2.length > 1) ? split2[1].trim() : "";
                if (!trim.startsWith("com.sun.management.")) {
                    error("agent.err.invalid.option", trim);
                }
                properties.setProperty(trim, value);
            }
            return properties;
        }
        return properties;
    }
    
    public static void premain(String s) throws Exception {
        agentmain(s);
    }
    
    public static void agentmain(String s) throws Exception {
        if (s == null || s.length() == 0) {
            s = "com.sun.management.jmxremote";
        }
        Properties string = parseString(s);
        Properties o = new Properties();
        readConfiguration(string.getProperty("com.sun.management.config.file"), o);
        o.putAll(string);
        startAgent(o);
    }
    
    private static synchronized void startLocalManagementAgent() {
        Properties agentProperties = VMSupport.getAgentProperties();
        if (agentProperties.get("com.sun.management.jmxremote.localConnectorAddress") == null) {
            String string = ConnectorBootstrap.startLocalConnectorServer().getAddress().toString();
            agentProperties.put("com.sun.management.jmxremote.localConnectorAddress", string);
            try {
                ConnectorAddressLink.export(string);
            }
            catch (Exception ex) {
                warning("agent.err.exportaddress.failed", ex.getMessage());
            }
        }
    }
    
    private static synchronized void startRemoteManagementAgent(String s) throws Exception {
        if (Agent.jmxServer != null) {
            throw new RuntimeException(getText("agent.err.invalid.state", new String[] { "Agent already started" }));
        }
        try {
            Properties string = parseString(s);
            Properties props = new Properties();
            readConfiguration(System.getProperty("com.sun.management.config.file"), props);
            Properties properties = System.getProperties();
            synchronized (properties) {
                props.putAll(properties);
            }
            String property = string.getProperty("com.sun.management.config.file");
            if (property != null) {
                readConfiguration(property, props);
            }
            props.putAll(string);
            if (props.getProperty("com.sun.management.enableThreadContentionMonitoring") != null) {
                ManagementFactory.getThreadMXBean().setThreadContentionMonitoringEnabled(true);
            }
            String property2 = props.getProperty("com.sun.management.jmxremote.port");
            if (property2 != null) {
                Agent.jmxServer = ConnectorBootstrap.startRemoteConnectorServer(property2, props);
                startDiscoveryService(props);
            }
            else {
                throw new AgentConfigurationError("agent.err.invalid.jmxremote.port", new String[] { "No port specified" });
            }
        }
        catch (AgentConfigurationError agentConfigurationError) {
            error(agentConfigurationError);
        }
        return;
    }
    
    private static synchronized void stopRemoteManagementAgent() throws Exception {
        JdpController.stopDiscoveryService();
        if (Agent.jmxServer != null) {
            ConnectorBootstrap.unexportRegistry();
            Agent.jmxServer.stop();
            Agent.jmxServer = null;
        }
    }
    
    private static void startAgent(Properties props) throws Exception {
        String property = props.getProperty("com.sun.management.snmp.port");
        String property2 = props.getProperty("com.sun.management.jmxremote");
        String property3 = props.getProperty("com.sun.management.jmxremote.port");
        if (props.getProperty("com.sun.management.enableThreadContentionMonitoring") != null) {
            ManagementFactory.getThreadMXBean().setThreadContentionMonitoringEnabled(true);
        }
        try {
            if (property != null) {
                loadSnmpAgent(property, props);
            }
            if (property2 != null || property3 != null) {
                if (property3 != null) {
                    Agent.jmxServer = ConnectorBootstrap.startRemoteConnectorServer(property3, props);
                    startDiscoveryService(props);
                }
                startLocalManagementAgent();
            }
        }
        catch (AgentConfigurationError agentConfigurationError) {
            error(agentConfigurationError);
        }
        catch (Exception ex) {
            error(ex);
        }
    }
    
    private static void startDiscoveryService(Properties properties) throws IOException {
        String property = properties.getProperty("com.sun.management.jdp.port");
        String property2 = properties.getProperty("com.sun.management.jdp.address");
        String property3 = properties.getProperty("com.sun.management.jmxremote.autodiscovery");
        int boolean1;
        if (property3 == null) {
            boolean1 = (property != null);
        }
        else {
            try {
                boolean1 = Boolean.parseBoolean(property3);
            }
            catch (NumberFormatException ex) {
                throw new AgentConfigurationError("Couldn't parse autodiscovery argument");
            }
        }
        if (!(boolean1 == 0)) {
            InetAddress address;
            try {
                address = ((property2 == null) ? InetAddress.getByName("224.0.23.178") : InetAddress.getByName(property2));
            }
            catch (UnknownHostException ex2) {
                throw new AgentConfigurationError("Unable to broadcast to requested address", ex2);
            }
            int int1 = 7095;
            if (property != null) {
                try {
                    int1 = Integer.parseInt(property);
                }
                catch (NumberFormatException ex3) {
                    throw new AgentConfigurationError("Couldn't parse JDP port argument");
                }
            }
            String property4 = properties.getProperty("com.sun.management.jmxremote.port");
            String property5 = properties.getProperty("com.sun.management.jmxremote.rmi.port");
            String host = Agent.jmxServer.getAddress().getHost();
            String url = (property5 != null) ? String.format("service:jmx:rmi://%s:%s/jndi/rmi://%s:%s/jmxrmi", new Object[] { host, property5, host, property4 }) : String.format("service:jmx:rmi:///jndi/rmi://%s:%s/jmxrmi", new Object[] { host, property4 });
            String property6 = properties.getProperty("com.sun.management.jdp.name");
            try {
                JdpController.startDiscoveryService(address, int1, property6, url);
            }
            catch (JdpException ex4) {
                throw new AgentConfigurationError("Couldn't start JDP service", ex4);
            }
        }
    }
    
    public static Properties loadManagementProperties() {
        Properties properties = new Properties();
        readConfiguration(System.getProperty("com.sun.management.config.file"), properties);
        Properties properties2 = System.getProperties();
        synchronized (properties2) {
            properties.putAll(properties2);
        }
        return properties;
    }
    
    public static synchronized Properties getManagementProperties() {
        if (Agent.mgmtProps == null) {
            String property = System.getProperty("com.sun.management.config.file");
            String property2 = System.getProperty("com.sun.management.snmp.port");
            String property3 = System.getProperty("com.sun.management.jmxremote");
            String property4 = System.getProperty("com.sun.management.jmxremote.port");
            if (property == null && property2 == null && property3 == null && property4 == null) {
                return null;
            }
            Agent.mgmtProps = loadManagementProperties();
        }
        return Agent.mgmtProps;
    }
    
    private static void loadSnmpAgent(String s, Properties properties) {
        try {
            Class.forName("sun.management.snmp.AdaptorBootstrap", true, null).getMethod("initialize", new Class[] { String.class, Properties.class }).invoke(null, new Object[] { s, properties });
        }
        catch (ClassNotFoundException | NoSuchMethodException | IllegalAccessException cause) {
            throw new UnsupportedOperationException("Unsupported management property: com.sun.management.snmp.port", cause);
        }
        catch (InvocationTargetException ex) {
            Throwable cause2 = ex.getCause();
            if ((cause2 instanceof RuntimeException)) {
                throw (RuntimeException)cause2;
            }
            if ((cause2 instanceof Error)) {
                throw (Error)cause2;
            }
            throw new UnsupportedOperationException("Unsupported management property: com.sun.management.snmp.port", cause2);
        }
    }
    
    private static void readConfiguration(String string, Properties properties) {
        if (string == null) {
            String property = System.getProperty("java.home");
            if (property == null) {
                throw new Error("Can't find java.home ??");
            }
            StringBuffer sb = new StringBuffer(property);
            sb.append(File.separator).append("lib");
            sb.append(File.separator).append("management");
            sb.append(File.separator).append("management.properties");
            string = sb.toString();
        }
        File file = new File(string);
        if (!file.exists()) {
            error("agent.err.configfile.notfound", string);
        }
        Object in = null;
        try {
            in = new FileInputStream(file);
            properties.load(new BufferedInputStream(in));
        }
        catch (FileNotFoundException ex) {
            error("agent.err.configfile.failed", ex.getMessage());
        }
        catch (IOException ex2) {
            error("agent.err.configfile.failed", ex2.getMessage());
        }
        catch (SecurityException ex3) {
            error("agent.err.configfile.access.denied", string);
        }
        finally {
            if (in != null) {
                try {
                    in.close();
                }
                catch (IOException ex4) {
                    error("agent.err.configfile.closed.failed", string);
                }
            }
        }
    }
    
    public static void startAgent() throws Exception {
        String property = System.getProperty("com.sun.management.agent.class");
        if (property == null) {
            Properties managementProperties = getManagementProperties();
            if (managementProperties != null) {
                startAgent(managementProperties);
            }
            return;
        }
        else {
            String[] split = property.split(":");
            if (split.length < 1 || split.length > 2) {
                error("agent.err.invalid.agentclass", "\"" + property + "\"");
            }
            String str = split[0];
            String s = (split.length == 2) ? split[1] : null;
            if (str == null || str.length() == 0) {
                error("agent.err.invalid.agentclass", "\"" + property + "\"");
            }
            if (str != null) {
                try {
                    ClassLoader.getSystemClassLoader().loadClass(str).getMethod("premain", new Class[] { String.class }).invoke(null, new Object[] { s });
                }
                catch (ClassNotFoundException ex) {
                    error("agent.err.agentclass.notfound", "\"" + str + "\"");
                }
                catch (NoSuchMethodException ex2) {
                    error("agent.err.premain.notfound", "\"" + str + "\"");
                }
                catch (SecurityException ex3) {
                    error("agent.err.agentclass.access.denied");
                }
                catch (Exception ex4) {
                    error("agent.err.agentclass.failed", (ex4.getCause() == null) ? ex4.getMessage() : ex4.getCause().getMessage());
                }
            }
            return;
        }
    }
    
    public static void error(String s) {
        String text = getText(s);
        System.err.print(getText("agent.err.error") + ": " + text);
        throw new RuntimeException(text);
    }
    
    public static void error(String s, String s2) {
        String text = getText(s);
        System.err.print(getText("agent.err.error") + ": " + text);
        System.err.println(": " + s2);
        throw new RuntimeException(text + ": " + s2);
    }
    
    public static void error(Exception cause) {
        cause.printStackTrace();
        System.err.println(getText("agent.err.exception") + ": " + cause.toString());
        throw new RuntimeException(cause);
    }
    
    public static void error(AgentConfigurationError cause) {
        String text = getText(cause.getError());
        String[] params = cause.getParams();
        System.err.print(getText("agent.err.error") + ": " + text);
        if (params != null && params.length != 0) {
            StringBuffer obj = new StringBuffer(params[0]);
            for (int i = 1; i < params.length; ++i) {
                obj.append(" " + params[i]);
            }
            System.err.println(": " + obj);
        }
        cause.printStackTrace();
        throw new RuntimeException(cause);
    }
    
    public static void warning(String s, String str) {
        System.err.print(getText("agent.err.warning") + ": " + getText(s));
        System.err.println(": " + str);
    }
    
    private static void initResource() {
        try {
            Agent.messageRB = ResourceBundle.getBundle("sun.management.resources.agent");
        }
        catch (MissingResourceException ex) {
            throw new Error("Fatal: Resource for management agent is missing");
        }
    }
    
    public static String getText(String s) {
        if (Agent.messageRB == null) {
            initResource();
        }
        try {
            return Agent.messageRB.getString(s);
        }
        catch (MissingResourceException ex) {
            return "Missing management agent resource bundle: key = \"" + s + "\"";
        }
    }
    
    public static String getText(String s, String... array) {
        if (Agent.messageRB == null) {
            initResource();
        }
        String pattern = Agent.messageRB.getString(s);
        if (pattern == null) {
            pattern = "missing resource key: key = \"" + s + "\", arguments = \"{0}\", \"{1}\", \"{2}\"";
        }
        return MessageFormat.format(pattern, (Object[])array);
    }
    
    static {
        Agent.jmxServer = null;
    }
}
